const express = require('express');
const cors = require('cors');
const { MongoClient, ServerApiVersion, ObjectId } = require('mongodb');
const dotenv = require('dotenv');
dotenv.config();
const app = express();
const port = process.env.PORT || 5000;
const multer = require('multer');
const fs = require('fs');
const path = require('path');
const { GalleryModel } = require('./models/modelGallery.');


// middleware 
app.use(cors());
app.use(express.json());

// Set up static file serving for the uploads folder
app.use('/uploads', express.static(path.join(__dirname, 'uploads')));

console.log(process.env.DB_USER)

// ensuring upload fodler exists
const uploadDir = "./uploads";
if (!fs.existsSync(uploadDir)) {
  fs.mkdir(uploadDir);
}

// multer configuration
// Multer storage configuration to preserve file extensions
const storage = multer.diskStorage({
  destination: function (req, file, cb) {
    cb(null, './uploads'); // Upload destination
  },
  filename: function (req, file, cb) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
    const extension = path.extname(file.originalname); // Get the file extension
    cb(null, `image-${uniqueSuffix}${extension}`);  // Save file with extension
  }
});

const upload = multer({ storage: storage })


const uri = `mongodb+srv://${process.env.DB_USER}:${process.env.DB_PASS}@cluster1.bqbyf.mongodb.net/?retryWrites=true&w=majority&appName=Cluster1`


// Create a MongoClient with a MongoClientOptions object to set the Stable API version
const client = new MongoClient(uri, {
  serverApi: {
    version: ServerApiVersion.v1,
    strict: true,
    deprecationErrors: true,
  }
});



async function run() {
  try {

    // Connect the client to the server	(optional starting in v4.7)
    await client.connect();

    // database name 
    const nameDB = 'krishiMongoDB'


    // API ROUTES 
    const newsRoute = "/news"
    const newsContentRoute = "/newsContent"
    const galleryRoute = "/gallery"

    // ImageHosting
    const imageHostingRoute = "/imageAPI"


    const subscriberRoute = "/subscriber"
    const messagesRoute = "/messages"
    const uploadRoute = "/uploadsAPI"

    // API COLLECTIONS
    const newsCollection = client.db(nameDB).collection('newsAPI');
    const subscriberCollection = client.db(nameDB).collection('subscriberAPI');
    const messagesCollection = client.db(nameDB).collection('messagesAPI');

    const imageCollection = client.db(nameDB).collection('imageAPI');

    const newsContentCollection = client.db(nameDB).collection('newsContentAPI');
    const galleryCollection = client.db(nameDB).collection('galleryAPI');
    const uploadCollection = client.db(nameDB).collection('uploadsAPI');


    // remaining APIS
    // newsDetailAPI
    // newsGalleryImageAPI



    //  Crud Operations

    // 1) Get data through app.get

    const GetRoute = (app, path, collection, sortField = '_id') => {
      app.get(path, async (req, res) => {
        try {
          const result = await collection.aggregate([
            {
              $addFields: {
                dateAsDate: { $toDate: "$date" }
              }
            },
            {
              $sort: { dateAsDate: -1 }
            }
          ]).toArray();
          res.send(result);
        } catch (error) {
          res.status(500).send({ error: 'Failed to fetch data' });
        }
      });
    };


    GetRoute(app, newsRoute, newsCollection, 'date');
    GetRoute(app, subscriberRoute, subscriberCollection);
    GetRoute(app, messagesRoute, messagesCollection, 'date');
    GetRoute(app, newsContentRoute, newsContentCollection);
    GetRoute(app, galleryRoute, galleryCollection);
    GetRoute(app, uploadRoute, uploadCollection);
    GetRoute(app, imageHostingRoute, imageCollection, 'date');



    // 2) Get Specific Data
    // get a specific product object
    const specificGetRoute = (app, path, collection) => {
      app.get(`${path}/:id`, async (req, res) => {
        const id = req.params.id;
        const query = { _id: new ObjectId(id) };
        const result = await collection.findOne(query);
        res.send(result);
      });
    };

    // edit from here

    specificGetRoute(app, newsRoute, newsCollection);
    specificGetRoute(app, subscriberRoute, subscriberCollection);
    specificGetRoute(app, messagesRoute, messagesCollection);


    const specificGetStringRoute = (app, path, collection) => {
      app.get(`${path}/:id`, async (req, res) => {
        try {
          const id = req.params.id;

          // Query by _id as a string
          const result = await collection.findOne({ _id: id });

          if (result) {
            res.send(result);
          } else {
            res.status(404).send({ error: 'Document not found' });
          }
        } catch (error) {
          console.error('Error fetching document:', error);
          res.status(500).send({ error: 'Error fetching document' });
        }
      });
    };



    specificGetStringRoute(app, newsContentRoute, newsContentCollection);
    specificGetStringRoute(app, galleryRoute, galleryCollection);







    // 3) Post data through app.post
    const postSingleRoute = (app, path, collection) => {
      app.post(path, async (req, res) => {
        try {
          console.log("Request Body:", req.body);  // Log the incoming request body

          // Insert the data into the specified collection
          const result = await collection.insertOne(req.body);

          // Log the result from MongoDB
          console.log("MongoDB Result:", result);

          // Send the result and insertedId back to the client
          res.send({
            insertedId: result.insertedId,
            ...result,  // Optional: Include additional MongoDB result details
          });
        } catch (error) {
          // Handle errors and send a 500 response
          console.error('Error inserting document:', error);
          res.status(500).send({ error: 'Failed to insert document' });
        }
      });
    };


    // Example usage
    postSingleRoute(app, newsRoute, newsCollection);
    postSingleRoute(app, subscriberRoute, subscriberCollection);
    postSingleRoute(app, messagesRoute, messagesCollection);
    postSingleRoute(app, imageHostingRoute, imageCollection);

    postSingleRoute(app, newsContentRoute, newsContentCollection);
    postSingleRoute(app, galleryRoute, galleryCollection);



    app.post(uploadRoute, upload.single('image'), async (req, res) => {
      try {
        // Check if file was uploaded
        if (!req.file) {
          return res.status(400).json({ error: 'No file uploaded' });
        }

        // Format the relative path for response
        const relativePath = `/uploads/${req.file.filename}`;
        const { filename } = req.file;

        // Log the relative path and filename
        console.log('Received file:', { filename, path: relativePath });

        // Respond with the relative path in JSON format
        res.json({ relativePath });

        // Log successful upload (for debugging purposes)
        console.log('Image successfully uploaded:', { filename, relativePath });

      } catch (error) {
        // Handle errors and send a 500 response
        console.error('Error uploading image:', error);
        res.status(500).json({ error: 'Failed to upload image' });
      }
    });






    // Danger (Very Careful)
    const deleteAll = (app, path, collection) => {
      app.delete(path, async (req, res) => {
        try {
          const result = await collection.deleteMany({}); // Deletes all items
          if (result.deletedCount > 0) {
            res.send({ message: `All documents in ${path} have been deleted.` });
          } else {
            res.send({ message: `No documents found in ${path} to delete.` });
          }
        } catch (error) {
          res.status(500).send({ error: `Failed to delete data from ${path}. Error: ${error.message}` });
        }
      });
    };

    deleteAll(app, newsRoute, newsCollection);
    deleteAll(app, subscriberRoute, subscriberCollection);
    deleteAll(app, messagesRoute, messagesCollection);

    deleteAll(app, newsContentRoute, newsContentCollection);
    deleteAll(app, galleryRoute, galleryCollection);

    // delete Specific
    // Delete News by ObjectId
    app.delete('/news/:id', async (req, res) => {
      const id = req.params.id;
      console.log('Deleting news with ID:', id); // Log the ID being deleted
      console.log('Request body:', req.body); // Log the request body

      try {
        const result = await newsCollection.deleteOne({ _id: new ObjectId(id) });

        if (result.deletedCount > 0) {
          res.status(200).json({ message: "News deleted successfully" });
        } else {
          res.status(404).json({ message: "News not found" });
        }
      } catch (error) {
        console.error('Error deleting news:', error);
        res.status(500).json({ error: "An error occurred while deleting the news" });
      }
    });

    app.delete('/newsContent/:id', async (req, res) => {
      const id = req.params.id; // Simple string ID
      console.log('Deleting news content with ID:', id); // Log the ID being deleted
      console.log('Request body:', req.body); // Log the request body

      try {
        const result = await newsContentCollection.deleteOne({ _id: id }); // Use string-based _id

        if (result.deletedCount > 0) {
          res.status(200).json({ message: "News content deleted successfully" });
        } else {
          res.status(404).json({ message: "News content not found" });
        }
      } catch (error) {
        console.error('Error deleting news content:', error);
        res.status(500).json({ error: "An error occurred while deleting the news content" });
      }
    });

    app.delete('/gallery/:id', async (req, res) => {
      const id = req.params.id;
      console.log('Received DELETE request for ID:', id); // Log incoming request
  
      try {
          const result = await galleryCollection.deleteOne({ _id: id });
          if (result.deletedCount > 0) {
              res.status(200).json({ message: "Gallery deleted successfully" });
          } else {
              res.status(404).json({ message: "Gallery not found" });
          }
      } catch (error) {
          console.error('Error deleting gallery:', error);
          res.status(500).json({ error: "An error occurred while deleting the gallery" });
      }
  });




    // Update News
    app.put('/news/:id', async (req, res) => {
      const id = req.params.id;

      try {
        const result = await newsCollection.updateOne(
          { _id: new ObjectId(id) },
          { $set: req.body }
        );

        const message = result.modifiedCount > 0 ? "News updated successfully" : "No changes detected.";
        console.log(message);
        if (result.modifiedCount > 0) console.log('Updated values:', req.body);

        res.status(200).json({ message });
      } catch (error) {
        console.error('Error updating news:', error);
        res.status(500).json({ error: "An error occurred while updating the news" });
      }
    });
    // Update News Content
    app.put('/newsContent/:id', async (req, res) => {
      const id = req.params.id; // Simple string ID

      try {
        const result = await newsContentCollection.updateOne(
          { _id: id }, // Use string-based _id
          { $set: req.body }
        );

        const message = result.modifiedCount > 0 ? "News content updated successfully" : "No changes detected.";
        console.log(message);
        if (result.modifiedCount > 0) console.log('Updated values:', req.body);

        res.status(200).json({ message });
      } catch (error) {
        console.error('Error updating news content:', error);
        res.status(500).json({ error: "An error occurred while updating the news content" });
      }
    });

    // Update Gallery
    app.put('/gallery/:id', async (req, res) => {
      const id = req.params.id; // Simple string ID

      try {
        const result = await galleryCollection.updateOne(
          { _id: id }, // Use string-based _id
          { $set: req.body }
        );

        const message = result.modifiedCount > 0 ? "Gallery updated successfully" : "No changes detected.";
        console.log(message);
        if (result.modifiedCount > 0) console.log('Updated values:', req.body);

        res.status(200).json({ message });
      } catch (error) {
        console.error('Error updating gallery:', error);
        res.status(500).json({ error: "An error occurred while updating the gallery" });
      }
    });



    // Send a ping to confirm a successful connection
    await client.db("admin").command({ ping: 1 });
    console.log("Pinged your deployment. You successfully connected to MongoDB!");
  } finally {
    // Ensures that the client will close when you finish/error
    // await client.close();
  }
}
run().catch(console.dir);




app.get('/', (req, res) => {
  res.send('Krishi Mongo Server is Running')
})

app.listen(port, () => {
  console.log(`Server is running on port http://localhost:${port}`)
})